# HG changeset patch
# User Günther Brammer <gbrammer@gmx.de>
# Date 1268692256 -3600
# Node ID 208ef23f2d7ed74d03402ae1e3fa9ec632a6252e
# Parent  4f6e7884ce7ff7b9551793f52607a218e5f74e06
autotools: Also check for the backtrace function before using execinfo.h

Also, install the crash handler even without backtrace. That way, at least
the signal name gets written to the log.

diff -r 4f6e7884ce7f -r 208ef23f2d7e config.h.in
--- a/config.h.in	Mo Mär 15 22:50:11 2010 +0100
+++ b/config.h.in	Mo Mär 15 23:30:56 2010 +0100
@@ -6,7 +6,7 @@
 /* Define to 1 if you have the <direct.h> header file. */
 #undef HAVE_DIRECT_H
 
-/* Define to 1 if you have the <execinfo.h> header file. */
+/* The backtrace function is declared in execinfo.h and works */
 #undef HAVE_EXECINFO_H
 
 /* Define to 1 if you have FreeType2. */
diff -r 4f6e7884ce7f -r 208ef23f2d7e configure.ac
--- a/configure.ac	Mo Mär 15 22:50:11 2010 +0100
+++ b/configure.ac	Mo Mär 15 23:30:56 2010 +0100
@@ -56,13 +56,27 @@
 
 # various used headers
 dnl the whitespace is there to prevent AC_INCLUDES_DEFAULT
-AC_CHECK_HEADERS([stdint.h unistd.h poll.h sys/stat.h sys/types.h locale.h sys/socket.h signal.h langinfo.h execinfo.h sys/inotify.h sys/syscall.h], , , [[ ]])
+AC_CHECK_HEADERS([stdint.h unistd.h poll.h sys/stat.h sys/types.h locale.h sys/socket.h signal.h langinfo.h sys/inotify.h sys/syscall.h], , , [[ ]])
 # Mingw does not ship with multimon.h
 AC_CHECK_HEADERS([multimon.h io.h direct.h share.h], [], [], [[#include <windows.h>]])
 # iconv
 AX_ICONV
 # vasprintf is a GNU extension
 AC_CHECK_FUNCS(vasprintf)
+# so is execinfo.h - and some systems have the header despite the functions not being in the c library
+AC_CHECK_HEADER(execinfo.h)
+if test "x$ac_cv_header_execinfo_h" = xyes; then
+  AC_CHECK_FUNC(backtrace, [], [
+    AC_CHECK_LIB(execinfo, backtrace, [
+      CLONK_LIBS="-lexecinfo $CLONK_LIBS"
+    ], [
+      ac_cv_header_execinfo_h=no
+    ])
+  ])
+fi
+if test "x$ac_cv_header_execinfo_h" = xyes; then
+  AC_DEFINE(HAVE_EXECINFO_H, 1, [The backtrace function is declared in execinfo.h and works])
+fi
 
 AC_ARG_ENABLE([debug],
   [AC_HELP_STRING([--enable-debug], [debugging options [default=no]])],
diff -r 4f6e7884ce7f -r 208ef23f2d7e src/C4WinMain.cpp
--- a/src/C4WinMain.cpp	Mo Mär 15 22:50:11 2010 +0100
+++ b/src/C4WinMain.cpp	Mo Mär 15 23:30:56 2010 +0100
@@ -131,7 +131,6 @@
 
 #include <unistd.h>
 #include <fcntl.h>
-#include <signal.h>
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -143,8 +142,11 @@
 # include <gtk/gtkwindow.h>
 #endif
 
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
 #ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
+#endif
 
 static void crash_handler(int signo)
 	{
@@ -170,6 +172,7 @@
 		else break;
 		if (logfd < 0) break;
 		}
+#ifdef HAVE_EXECINFO_H
 	// Get the backtrace
 	void *stack[100];
 	int count = backtrace(stack, 100);
@@ -178,10 +181,11 @@
 	// Also to the log file
 	if (logfd >= 0)
 		backtrace_symbols_fd (stack, count, logfd);
+#endif
 	// Bye.
 	_exit(C4XRV_Failure);
 	}
-#endif
+#endif // HAVE_SIGNAL_H
 
 #ifdef __APPLE__
 void restart(char* args[]) {
@@ -206,7 +210,7 @@
 		printf("Do not run %s as root!\n", argc ? argv[0] : "this program");
 		return C4XRV_Failure;
 		}
-#ifdef HAVE_EXECINFO_H
+#ifdef HAVE_SIGNAL_H
 	// Set up debugging facilities
 	signal(SIGBUS, crash_handler);
 	signal(SIGILL, crash_handler);
